@@ -1,6 +1,6 @@
 #! /bin/bash
 ################################################################################
-# Copyright 2013-2014 Aerospike, Inc.
+# Copyright 2013-2015 Aerospike, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,6 +16,8 @@
 ################################################################################
 
 AEROSPIKE_C_VERSION=${AEROSPIKE_C_VERSION:-'latest'}
+# always download unless environment var is set to 0, then check locally first
+DOWNLOAD=${DOWNLOAD_C_CLIENT:-1}
 
 ################################################################################
 #
@@ -51,26 +53,41 @@ detect_linux()
 
     case ${DIST_NAME} in
 
-      "centos6" | "redhatenterpriceserver6" | "fedora20" | "oracleserver6" )
+      "centos6" | "centos7" | "redhatenterpriseserver6" | "redhatenterpriseserver7" | "fedora20" | "fedora21" | "fedora22" | "oracleserver6" | "scientific6" )
         echo "el6" "rpm"
         return 0
         ;;
 
-      "debian6" | "debian7" )
-        echo ${DIST_NAME} "deb"
+      "debian6" )
+        echo "debian6" "deb"
         return 0
         ;;
 
-      "ubuntu12" | "ubuntu13" | "ubuntu14" )
+      "debian7" | "debian8" )
+        echo "debian7" "deb"
+        return 0
+        ;;
+
+      "ubuntu12" | "ubuntu13" )
         echo "ubuntu12"  "deb"
         return 0
         ;;
 
-      "linuxmint17" )
-        echo "ubuntu12"  "deb"
+      "ubuntu14" | "ubuntu15" | "linuxmint17" )
+        echo "ubuntu14"  "deb"
         return 0
         ;;
 
+      "ubuntu16" | "linuxmint18" )
+        echo "ubuntu16" "deb"
+        return 0
+        ;;
+
+      "amazonami"* )
+        echo "el6" "rpm"
+    	return 0
+    	;;
+
       * )
         echo "error: ${DIST_NAME} is not supported."
         return 1
@@ -106,7 +123,7 @@ detect_linux()
     dist=$(cat /etc/issue | tr '[:upper:]' '[:lower:]')
     case ${dist} in
 
-      "centos"* | "red hat enterprise linux"* | "fedora"* | "oracleserver"* )
+      "centos"* | "red hat enterprise linux"* | "fedora"* | "oracleserver"* | "scientific linux"* )
         echo "el6" "rpm"
         return 0
         ;;
@@ -124,6 +141,11 @@ detect_linux()
             return 0
             ;;
 
+          "8."* )
+            echo "debian7" "deb"
+            return 0
+            ;;
+
           * )
             echo "error: Debian ${vers} is not supported."
             return 1
@@ -134,11 +156,21 @@ detect_linux()
       "ubuntu"* )
         vers=$(lsb_release -r -s)
         case ${vers} in
-          "12."* | "13."* | "14.*" )
+          "12."* | "13."* )
             echo "ubuntu12"  "deb"
             return 0
             ;;
 
+          "14."* | "15."* )
+            echo "ubuntu14"  "deb"
+            return 0
+            ;;
+
+          "16."* )
+            echo "ubuntu16" "deb"
+            return 0
+            ;;
+
           * )
             echo "error: Ubuntu ${vers} is not supported."
             return 1
@@ -176,7 +208,7 @@ download()
   wget_path=`which wget`
   has_wget=$?
 
-  # Check 
+  # Check
   if [ $has_curl != 0 ] && [ $has_wget != 0 ]; then
     echo "error: Not able to find 'curl' or `wget`. Either is required to download the package."
     exit 1
@@ -292,7 +324,6 @@ if [ $DOWNLOAD ] && [ $DOWNLOAD == 1 ]; then
     printf "warning: new package. If you would like to download a new package\n"
     printf "warning: then please remove the 'aerospike-client' directory and any \n"
     printf "warning: 'aerospike-client.tgz' file in this directory.\n"
-    printf "warning: \n"
   else
 
     ##############################################################################
@@ -305,7 +336,6 @@ if [ $DOWNLOAD ] && [ $DOWNLOAD == 1 ]; then
       printf "warning: We will be using this package, rather than downloading a new package.\n"
       printf "warning: If you would like to download a new package, then please remove.\n"
       printf "warning: 'aerospike-client.tgz' from this directory.\n"
-      printf "warning: \n"
     else
       download
       
