@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2014 Aerospike, Inc.
+ * Copyright 2013-2016 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,12 +24,23 @@
 
 #include "client.h"
 #include "conversions.h"
-#include "key.h"
+#include "exceptions.h"
 #include "policy.h"
 
-PyObject * AerospikeClient_Exists_Invoke(
-	AerospikeClient * self,
-	PyObject * py_key, PyObject * py_policy)
+/**
+ *******************************************************************************************************
+ * This function applies a registered udf module on a particular record.
+ *
+ * @param self                  AerospikeClient object
+ * @param py_key                The key under which the record is stored.
+ * @param py_policy             The dictionary of policies
+ *
+ * Returns a tuple of record having key and meta sequentially.
+ *******************************************************************************************************
+ */
+extern PyObject * AerospikeClient_Exists_Invoke(
+		AerospikeClient * self,
+		PyObject * py_key, PyObject * py_policy)
 {
 	// Python Return Value
 	PyObject * py_result = NULL;
@@ -41,6 +52,9 @@ PyObject * AerospikeClient_Exists_Invoke(
 	as_key key;
 	as_record * rec = NULL;
 
+	// Initialisation flags
+	bool key_initialised = false;
+
 	// Initialize error
 	as_error_init(&err);
 
@@ -49,23 +63,32 @@ PyObject * AerospikeClient_Exists_Invoke(
 		goto CLEANUP;
 	}
 
-	// Convert python key object to as_key
-	pyobject_to_key(&err, py_key, &key);
-	if ( err.code != AEROSPIKE_OK ) {
+	if (!self->is_conn_16) {
+		as_error_update(&err, AEROSPIKE_ERR_CLUSTER, "No connection to aerospike cluster");
 		goto CLEANUP;
 	}
 
+	// Convert python key object to as_key
+	pyobject_to_key(&err, py_key, &key);
+	if (err.code != AEROSPIKE_OK) {
+		goto CLEANUP;
+	}
+	// key is initialised successfully
+	key_initialised = true;
+
 	// Convert python policy object to as_policy_exists
-	pyobject_to_policy_read(&err, py_policy, &read_policy, &read_policy_p);
-	if ( err.code != AEROSPIKE_OK ) {
+	pyobject_to_policy_read(&err, py_policy, &read_policy, &read_policy_p,
+			&self->as->config.policies.read);
+	if (err.code != AEROSPIKE_OK) {
 		goto CLEANUP;
 	}
 
 	// Invoke operation
+	Py_BEGIN_ALLOW_THREADS
 	aerospike_key_exists(self->as, &err, read_policy_p, &key, &rec);
+	Py_END_ALLOW_THREADS
 
-	if ( err.code == AEROSPIKE_OK ) {
-
+	if (err.code == AEROSPIKE_OK) {
 		PyObject * py_result_key = NULL;
 		PyObject * py_result_meta = NULL;
 
@@ -75,8 +98,7 @@ PyObject * AerospikeClient_Exists_Invoke(
 		py_result = PyTuple_New(2);
 		PyTuple_SetItem(py_result, 0, py_result_key);
 		PyTuple_SetItem(py_result, 1, py_result_meta);
-	}
-	else if ( err.code == AEROSPIKE_ERR_RECORD_NOT_FOUND ) {
+	} else if (err.code == AEROSPIKE_ERR_RECORD_NOT_FOUND) {
 		as_error_reset(&err);
 
 		PyObject * py_result_key = NULL;
@@ -90,22 +112,50 @@ PyObject * AerospikeClient_Exists_Invoke(
 
 		Py_INCREF(py_result_meta);
 	}
+	else {
+		as_error_update(&err, err.code, NULL);
+	}
 
 CLEANUP:
 
-	as_record_destroy(rec);
+	if (key_initialised == true) {
+		// Destroy the key if it is initialised successfully.
+		as_key_destroy(&key);
+	}
+	if (rec) {
+		as_record_destroy(rec);
+	}
 
-	if ( err.code != AEROSPIKE_OK ) {
+	if (err.code != AEROSPIKE_OK) {
 		PyObject * py_err = NULL;
 		error_to_pyobject(&err, &py_err);
-		PyErr_SetObject(PyExc_Exception, py_err);
+		PyObject *exception_type = raise_exception(&err);
+		if (PyObject_HasAttrString(exception_type, "key")) {
+			PyObject_SetAttrString(exception_type, "key", py_key);
+		} 
+		if (PyObject_HasAttrString(exception_type, "bin")) {
+			PyObject_SetAttrString(exception_type, "bin", Py_None);
+		}
+		PyErr_SetObject(exception_type, py_err);
 		Py_DECREF(py_err);
-		return NULL;
 	}
 
 	return py_result;
 }
 
+/**
+ *******************************************************************************************************
+ * Checks if a record exists in the Aerospike DB.
+ *
+ * @param self                  AerospikeClient object
+ * @param args                  The args is a tuple object containing an argument
+ *                              list passed from Python to a C function
+ * @param kwds                  Dictionary of keywords
+ *
+ * Returns a tuple of record having key and meta sequentially.
+ * In case of error,appropriate exceptions will be raised.
+ *******************************************************************************************************
+ */
 PyObject * AerospikeClient_Exists(AerospikeClient * self, PyObject * args, PyObject * kwds)
 {
 	// Python Function Arguments
@@ -116,8 +166,8 @@ PyObject * AerospikeClient_Exists(AerospikeClient * self, PyObject * args, PyObj
 	static char * kwlist[] = {"key", "policy", NULL};
 
 	// Python Function Argument Parsing
-	if ( PyArg_ParseTupleAndKeywords(args, kwds, "O|O:exists", kwlist,
-			&py_key, &py_policy) == false ) {
+	if (PyArg_ParseTupleAndKeywords(args, kwds, "O|O:exists", kwlist,
+			&py_key, &py_policy) == false) {
 		return NULL;
 	}
 
