@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2014 Aerospike, Inc.
+ * Copyright 2013-2016 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,15 +24,28 @@
 
 #include "client.h"
 #include "conversions.h"
-#include "key.h"
+#include "exceptions.h"
 #include "policy.h"
 
+/**
+ *******************************************************************************************************
+ * This function projects bins on given namespace nd set.
+ *
+ * @param self                  AerospikeClient object
+ * @param py_key                The key under which to store the record.
+ * @param py_bins               The bins to project into the DB.
+ * @param py_policy             The optional policy parameters
+ *
+ * Returns a tuple containing key, meta and bins.
+ *******************************************************************************************************
+ */
 PyObject * AerospikeClient_Select_Invoke(
 	AerospikeClient * self,
 	PyObject * py_key, PyObject * py_bins, PyObject * py_policy)
 {
 	// Python Return Value
 	PyObject * py_rec = NULL;
+	PyObject * py_ustr = NULL;
 
 	// Aerospike Client Arguments
 	as_error err;
@@ -42,6 +55,9 @@ PyObject * AerospikeClient_Select_Invoke(
 	as_record * rec = NULL;
 	char ** bins = NULL;
 
+	// Initialisation flags
+	bool key_initialised = false;
+
 	// Initialize error
 	as_error_init(&err);
 
@@ -50,36 +66,49 @@ PyObject * AerospikeClient_Select_Invoke(
 		goto CLEANUP;
 	}
 
-	// Convert python key object to as_key
-	pyobject_to_key(&err, py_key, &key);
-	if ( err.code != AEROSPIKE_OK ) {
+	if (!self->is_conn_16) {
+		as_error_update(&err, AEROSPIKE_ERR_CLUSTER, "No connection to aerospike cluster");
 		goto CLEANUP;
 	}
 
+	// Convert python key object to as_key
+	pyobject_to_key(&err, py_key, &key);
+	if (err.code != AEROSPIKE_OK) {
+		goto CLEANUP;
+	}
+	// key is initialised successfully
+	key_initialised = true;
 
 	// Convert python bins list to char ** bins
-	if ( py_bins != NULL && PyList_Check(py_bins) ) {
+	if (py_bins && PyList_Check(py_bins)) {
 		Py_ssize_t size = PyList_Size(py_bins);
 		bins = (char **) alloca(sizeof(char *) * (size+1));
-		for ( int i = 0; i < size; i++ ) {
+		for (int i = 0; i < size; i++) {
 			PyObject * py_val = PyList_GetItem(py_bins, i);
 			bins[i] = (char *) alloca(sizeof(char) * AS_BIN_NAME_MAX_SIZE);
-			if ( PyString_Check(py_val) ) {
+			if (PyString_Check(py_val)) {
 				strncpy(bins[i], PyString_AsString(py_val), AS_BIN_NAME_MAX_LEN);
 				bins[i][AS_BIN_NAME_MAX_LEN] = '\0';
 			}
 		}
 		bins[size] = NULL;
 	}
-	else if ( py_bins != NULL && PyTuple_Check(py_bins) ) {
+	else if (py_bins && PyTuple_Check(py_bins)) {
 		Py_ssize_t size = PyTuple_Size(py_bins);
 		bins = (char **) alloca(sizeof(char *) * (size+1));
-		for ( int i = 0; i < size; i++ ) {
+		for (int i = 0; i < size; i++) {
 			PyObject * py_val = PyTuple_GetItem(py_bins, i);
 			bins[i] = (char *) alloca(sizeof(char) * AS_BIN_NAME_MAX_SIZE);
-			if ( PyString_Check(py_val) ) {
+			if (PyUnicode_Check(py_val)) {
+				py_ustr = PyUnicode_AsUTF8String(py_val);
+				strncpy(bins[i], PyBytes_AsString(py_ustr), AS_BIN_NAME_MAX_LEN);
+				bins[i][AS_BIN_NAME_MAX_LEN] = '\0';
+			} else if (PyString_Check(py_val)) {
 				strncpy(bins[i], PyString_AsString(py_val), AS_BIN_NAME_MAX_LEN);
 				bins[i][AS_BIN_NAME_MAX_LEN] = '\0';
+			} else {
+				as_error_update(&err, AEROSPIKE_ERR_PARAM, "not string type");
+				goto CLEANUP;
 			}
 		}
 		bins[size] = NULL;
@@ -90,8 +119,9 @@ PyObject * AerospikeClient_Select_Invoke(
 	}
 
 	// Convert python policy object to as_policy_exists
-	pyobject_to_policy_read(&err, py_policy, &read_policy, &read_policy_p);
-	if ( err.code != AEROSPIKE_OK ) {
+	pyobject_to_policy_read(&err, py_policy, &read_policy, &read_policy_p,
+			&self->as->config.policies.read);
+	if (err.code != AEROSPIKE_OK) {
 		goto CLEANUP;
 	}
 
@@ -99,38 +129,41 @@ PyObject * AerospikeClient_Select_Invoke(
 	as_record_init(rec, 0);
 
 	// Invoke operation
+	Py_BEGIN_ALLOW_THREADS
 	aerospike_key_select(self->as, &err, read_policy_p, &key, (const char **) bins, &rec);
+	Py_END_ALLOW_THREADS
 
-	if ( err.code == AEROSPIKE_OK ) {
-		record_to_pyobject(&err, rec, &key, &py_rec);
+	if (err.code == AEROSPIKE_OK) {
+		record_to_pyobject(self, &err, rec, &key, &py_rec);
 	}
-	else if ( err.code == AEROSPIKE_ERR_RECORD_NOT_FOUND ) {
-		as_error_reset(&err);
-
-		PyObject * py_rec_key = NULL;
-		PyObject * py_rec_meta = Py_None;
-		PyObject * py_rec_bins = Py_None;
-
-		key_to_pyobject(&err, &key, &py_rec_key);
-
-		py_rec = PyTuple_New(3);
-		PyTuple_SetItem(py_rec, 0, py_rec_key);
-		PyTuple_SetItem(py_rec, 1, py_rec_meta);
-		PyTuple_SetItem(py_rec, 2, py_rec_bins);
-
-		Py_INCREF(py_rec_meta);
-		Py_INCREF(py_rec_bins);
+	else {
+		as_error_update(&err, err.code, NULL);
 	}
 
 CLEANUP:
 
-	// as_key_destroy(&key);
+	if (py_ustr) {
+		Py_DECREF(py_ustr);
+	}
+
+	if (key_initialised == true) {
+		// Destroy the key if it is initialised successfully.
+		as_key_destroy(&key);
+	}
+
 	as_record_destroy(rec);
 
-	if ( err.code != AEROSPIKE_OK ) {
+	if (err.code != AEROSPIKE_OK) {
 		PyObject * py_err = NULL;
 		error_to_pyobject(&err, &py_err);
-		PyErr_SetObject(PyExc_Exception, py_err);
+		PyObject *exception_type = raise_exception(&err);
+		if (PyObject_HasAttrString(exception_type, "key")) {
+			PyObject_SetAttrString(exception_type, "key", py_key);
+		} 
+		if (PyObject_HasAttrString(exception_type, "bin")) {
+			PyObject_SetAttrString(exception_type, "bin", Py_None);
+		}
+		PyErr_SetObject(exception_type, py_err);
 		Py_DECREF(py_err);
 		return NULL;
 	}
@@ -138,6 +171,19 @@ CLEANUP:
 	return py_rec;
 }
 
+/**
+ *******************************************************************************************************
+ * Projects bins on given namespace and set.
+ *
+ * @param self                  AerospikeClient object
+ * @param args                  The args is a tuple object containing an argument
+ *                              list passed from Python to a C function
+ * @param kwds                  Dictionary of keywords
+ *
+ * Returns a tuple containing key, meta and bins.
+ * In case of error,appropriate exceptions will be raised.
+ *******************************************************************************************************
+ */
 PyObject * AerospikeClient_Select(AerospikeClient * self, PyObject * args, PyObject * kwds)
 {
 	// Python Function Arguments
@@ -149,8 +195,8 @@ PyObject * AerospikeClient_Select(AerospikeClient * self, PyObject * args, PyObj
 	static char * kwlist[] = {"key", "bins", "policy", NULL};
 
 	// Python Function Argument Parsing
-	if ( PyArg_ParseTupleAndKeywords(args, kwds, "OO|O:select", kwlist,
-			&py_key, &py_bins, &py_policy) == false ) {
+	if (PyArg_ParseTupleAndKeywords(args, kwds, "OO|O:select", kwlist,
+			&py_key, &py_bins, &py_policy) == false) {
 		return NULL;
 	}
 
