@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2014 Aerospike, Inc.
+ * Copyright 2013-2016 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,9 +21,11 @@
 #include <aerospike/as_error.h>
 #include <aerospike/as_policy.h>
 #include <aerospike/as_query.h>
+#include <aerospike/as_arraylist.h>
 
 #include "client.h"
 #include "conversions.h"
+#include "exceptions.h"
 #include "query.h"
 #include "policy.h"
 
@@ -31,6 +33,7 @@
 typedef struct {
 	as_error error;
 	PyObject * callback;
+	AerospikeClient * client;
 } LocalData;
 
 
@@ -38,7 +41,7 @@ static bool each_result(const as_val * val, void * udata)
 {
 	bool rval = true;
 
-	if ( !val ) {
+	if (!val) {
 		return false;
 	}
 
@@ -57,7 +60,7 @@ static bool each_result(const as_val * val, void * udata)
 	gstate = PyGILState_Ensure();
 
 	// Convert as_val to a Python Object
-	val_to_pyobject(err, val, &py_result);
+	val_to_pyobject(data->client, err, val, &py_result);
 
 	// Build Python Function Arguments
 	py_arglist = PyTuple_New(1);
@@ -69,14 +72,14 @@ static bool each_result(const as_val * val, void * udata)
 	// Release Python Function Arguments
 	Py_DECREF(py_arglist);
 	// handle return value
-	if ( py_return == NULL ) {
+	if (!py_return) {
 		// an exception was raised, handle it (someday)
 		// for now, we bail from the loop
-		as_error_update(err, AEROSPIKE_ERR_PARAM, "Callback function contains an error");
-		rval = false;
+		as_error_update(err, AEROSPIKE_ERR_CLIENT, "Callback function contains an error");
+		rval = true;
 	}
-	else if (  PyBool_Check(py_return) ) {
-		if ( Py_False == py_return ) {
+	else if (PyBool_Check(py_return)) {
+		if (Py_False == py_return) {
 			rval = false;
 		}
 		else {
@@ -105,11 +108,17 @@ PyObject * AerospikeQuery_Foreach(AerospikeQuery * self, PyObject * args, PyObje
 	static char * kwlist[] = {"callback", "policy", NULL};
 
 	// Python Function Argument Parsing
-	if ( PyArg_ParseTupleAndKeywords(args, kwds, "O|O:foreach", kwlist, &py_callback, &py_policy) == false ) {
+	if (PyArg_ParseTupleAndKeywords(args, kwds, "O|O:foreach", kwlist, &py_callback, &py_policy) == false) {
 		as_query_destroy(&self->query);
 		return NULL;
 	}
 
+	// Initialize callback user data
+	LocalData data;
+	data.callback = py_callback;
+	data.client = self->client;
+	as_error_init(&data.error);
+
 	// Aerospike Client Arguments
 	as_error err;
 	as_policy_query query_policy;
@@ -123,16 +132,17 @@ PyObject * AerospikeQuery_Foreach(AerospikeQuery * self, PyObject * args, PyObje
 		goto CLEANUP;
 	}
 
-	// Convert python policy object to as_policy_exists
-	pyobject_to_policy_query(&err, py_policy, &query_policy, &query_policy_p);
-	if ( err.code != AEROSPIKE_OK ) {
+	if (!self->client->is_conn_16) {
+		as_error_update(&err, AEROSPIKE_ERR_CLUSTER, "No connection to aerospike cluster");
 		goto CLEANUP;
 	}
 
-	// Create and initialize callback user-data
-	LocalData data;
-	data.callback = py_callback;
-	as_error_init(&data.error);
+	// Convert python policy object to as_policy_exists
+	pyobject_to_policy_query(&err, py_policy, &query_policy, &query_policy_p,
+			&self->client->as->config.policies.query);
+	if (err.code != AEROSPIKE_OK) {
+		goto CLEANUP;
+	}
 
 	// We are spawning multiple threads
 	PyThreadState * _save = PyEval_SaveThread();
@@ -143,16 +153,31 @@ PyObject * AerospikeQuery_Foreach(AerospikeQuery * self, PyObject * args, PyObje
 	// We are done using multiple threads
 	PyEval_RestoreThread(_save);
 	if (data.error.code != AEROSPIKE_OK) {
+		as_error_update(&data.error, data.error.code, NULL);
 		goto CLEANUP;
 	}
 
 CLEANUP:
+	if (self->query.apply.arglist) {
+		as_arraylist_destroy( (as_arraylist *) self->query.apply.arglist );
+	}
 	self->query.apply.arglist = NULL;
-	as_query_destroy(&self->query);
-	if ( err.code != AEROSPIKE_OK ) {
+
+	if (err.code != AEROSPIKE_OK || data.error.code != AEROSPIKE_OK) {
 		PyObject * py_err = NULL;
-		error_to_pyobject(&err, &py_err);
-		PyErr_SetObject(PyExc_Exception, py_err);
+		PyObject *exception_type = NULL;
+		if (err.code != AEROSPIKE_OK) {
+			error_to_pyobject(&err, &py_err);
+			exception_type = raise_exception(&err);
+		}
+		if (data.error.code != AEROSPIKE_OK) {
+			error_to_pyobject(&data.error, &py_err);
+			exception_type = raise_exception(&data.error);
+		}
+		if (PyObject_HasAttrString(exception_type, "name")) {
+			PyObject_SetAttrString(exception_type, "name", Py_None);
+		}
+		PyErr_SetObject(exception_type, py_err);
 		Py_DECREF(py_err);
 		return NULL;
 	}
