@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2014 Aerospike, Inc.
+ * Copyright 2013-2016 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 
 #include "client.h"
 #include "conversions.h"
+#include "exceptions.h"
 #include "scan.h"
 #include "policy.h"
 
@@ -30,6 +31,7 @@
 typedef struct {
 	as_error error;
 	PyObject * callback;
+	AerospikeClient * client;
 } LocalData;
 
 
@@ -37,7 +39,7 @@ static bool each_result(const as_val * val, void * udata)
 {
 	bool rval = true;
 
-	if ( !val ) {
+	if (!val) {
 		return false;
 	}
 
@@ -56,7 +58,7 @@ static bool each_result(const as_val * val, void * udata)
 	gstate = PyGILState_Ensure();
 
 	// Convert as_val to a Python Object
-	val_to_pyobject(err, val, &py_result);
+	val_to_pyobject(data->client, err, val, &py_result);
 
 	// Build Python Function Arguments
 	py_arglist = PyTuple_New(1);
@@ -69,14 +71,14 @@ static bool each_result(const as_val * val, void * udata)
 	Py_DECREF(py_arglist);
 
 	// handle return value
-	if ( py_return == NULL ) {
+	if (!py_return) {
 		// an exception was raised, handle it (someday)
 		// for now, we bail from the loop
-		as_error_update(err, AEROSPIKE_ERR_PARAM, "Callback function contains an error");
+		as_error_update(err, AEROSPIKE_ERR_CLIENT, "Callback function raised an exception");
 		rval = false;
 	}
-	else if (  PyBool_Check(py_return) ) {
-		if ( Py_False == py_return ) {
+	else if (PyBool_Check(py_return)) {
+		if (Py_False == py_return) {
 			rval = false;
 		}
 		else {
@@ -100,17 +102,24 @@ PyObject * AerospikeScan_Foreach(AerospikeScan * self, PyObject * args, PyObject
 	// Python Function Arguments
 	PyObject * py_callback = NULL;
 	PyObject * py_policy = NULL;
+	PyObject * py_options = NULL;
 	as_policy_scan scan_policy;
 	as_policy_scan * scan_policy_p = NULL;
 
 	// Python Function Keyword Arguments
-	static char * kwlist[] = {"callback", "policy", NULL};
+	static char * kwlist[] = {"callback", "policy", "options", NULL};
 
 	// Python Function Argument Parsing
-	if ( PyArg_ParseTupleAndKeywords(args, kwds, "O|O:foreach", kwlist, &py_callback, &py_policy) == false ) {
+	if (PyArg_ParseTupleAndKeywords(args, kwds, "O|OO:foreach", kwlist, &py_callback, &py_policy, &py_options) == false) {
 		return NULL;
 	}
 
+	// Create and initialize callback user-data
+	LocalData data;
+	data.callback = py_callback;
+	data.client = self->client;
+	as_error_init(&data.error);
+
 	// Aerospike Client Arguments
 	as_error err;
 
@@ -122,16 +131,23 @@ PyObject * AerospikeScan_Foreach(AerospikeScan * self, PyObject * args, PyObject
 		goto CLEANUP;
 	}
 
-	// Convert python policy object to as_policy_exists
-	pyobject_to_policy_scan(&err, py_policy, &scan_policy, &scan_policy_p);
-	if ( err.code != AEROSPIKE_OK ) {
+	if (!self->client->is_conn_16) {
+		as_error_update(&err, AEROSPIKE_ERR_CLUSTER, "No connection to aerospike cluster");
 		goto CLEANUP;
 	}
 
-	// Create and initialize callback user-data
-	LocalData data;
-	data.callback = py_callback;
-	as_error_init(&data.error);
+	// Convert python policy object to as_policy_exists
+	pyobject_to_policy_scan(&err, py_policy, &scan_policy, &scan_policy_p,
+			&self->client->as->config.policies.scan);
+	if (err.code != AEROSPIKE_OK) {
+		goto CLEANUP;
+	}
+	if (py_options && PyDict_Check(py_options)) {
+		set_scan_options(&err, &self->scan, py_options);
+		if (err.code != AEROSPIKE_OK) {
+			goto CLEANUP;
+		}
+	}
 
 	// We are spawning multiple threads
 	PyThreadState * _save = PyEval_SaveThread();
@@ -142,15 +158,23 @@ PyObject * AerospikeScan_Foreach(AerospikeScan * self, PyObject * args, PyObject
 	// We are done using multiple threads
 	PyEval_RestoreThread(_save);
 	if (data.error.code != AEROSPIKE_OK) {
+		as_error_update(&data.error, data.error.code, NULL);
 		goto CLEANUP;
 	}
 
 CLEANUP:
 
-	if ( err.code != AEROSPIKE_OK ) {
-		PyObject * py_err = NULL;
-		error_to_pyobject(&err, &py_err);
-		PyErr_SetObject(PyExc_Exception, py_err);
+	if (err.code != AEROSPIKE_OK || data.error.code != AEROSPIKE_OK) {
+		PyObject * py_err = NULL, *exception_type = NULL;
+		if (err.code != AEROSPIKE_OK){
+			error_to_pyobject(&err, &py_err);
+			exception_type = raise_exception(&err);
+		}
+		if (data.error.code != AEROSPIKE_OK){
+			error_to_pyobject(&data.error, &py_err);
+			exception_type = raise_exception(&data.error);
+		}
+		PyErr_SetObject(exception_type, py_err);
 		Py_DECREF(py_err);
 		return NULL;
 	}
