@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright 2013-2014 Aerospike, Inc.
+ * Copyright 2013-2016 Aerospike, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,18 +24,27 @@
 
 #include "client.h"
 #include "conversions.h"
+#include "exceptions.h"
+#include "policy.h"
 #include "scan.h"
 
 #undef TRACE
 #define TRACE()
 
+typedef struct {
+	PyObject * py_results;
+	AerospikeClient * client;
+} LocalData;
+
 static bool each_result(const as_val * val, void * udata)
 {
-	if ( !val ) {
+	if (!val) {
 		return false;
 	}
 
-	PyObject * py_results = (PyObject *) udata;
+	PyObject * py_results = NULL;
+	LocalData *data = (LocalData *) udata;
+	py_results = data->py_results;
 	PyObject * py_result = NULL;
 
 	as_error err;
@@ -43,9 +52,9 @@ static bool each_result(const as_val * val, void * udata)
 	PyGILState_STATE gstate;
 	gstate = PyGILState_Ensure();
 
-	val_to_pyobject(&err, val, &py_result);
+	val_to_pyobject(data->client, &err, val, &py_result);
 
-	if ( py_result ) {
+	if (py_result) {
 		PyList_Append(py_results, py_result);
 		Py_DECREF(py_result);
 	}
@@ -58,28 +67,56 @@ static bool each_result(const as_val * val, void * udata)
 PyObject * AerospikeScan_Results(AerospikeScan * self, PyObject * args, PyObject * kwds)
 {
 	PyObject * py_policy = NULL;
+	PyObject * py_results = NULL;
+	as_policy_scan scan_policy;
+	as_policy_scan * scan_policy_p = NULL;
 
+	LocalData data;
+	data.client = self->client;
 	static char * kwlist[] = {"policy", NULL};
 
-	if ( PyArg_ParseTupleAndKeywords(args, kwds, "|O:foreach", kwlist, &py_policy) == false ) {
+	if (PyArg_ParseTupleAndKeywords(args, kwds, "|O:results", kwlist, &py_policy) == false) {
 		return NULL;
 	}
 
 	as_error err;
 	as_error_init(&err);
 
-	PyObject * py_results = PyList_New(0);
+	if (!self || !self->client->as) {
+		as_error_update(&err, AEROSPIKE_ERR_PARAM, "Invalid aerospike object");
+		goto CLEANUP;
+	}
+	if (!self->client->is_conn_16) {
+		as_error_update(&err, AEROSPIKE_ERR_CLUSTER, "No connection to aerospike cluster");
+		goto CLEANUP;
+	}
+
+	// Convert python policy object to as_policy_scan
+	pyobject_to_policy_scan(&err, py_policy, &scan_policy, &scan_policy_p,
+			&self->client->as->config.policies.scan);
+	if (err.code != AEROSPIKE_OK) {
+		as_error_update(&err, err.code, NULL);
+		goto CLEANUP;
+	}
+
+	py_results = PyList_New(0);
+	data.py_results = py_results;
 
 	PyThreadState * _save = PyEval_SaveThread();
 
-	aerospike_scan_foreach(self->client->as, &err, NULL, &self->scan, each_result, py_results);
+	aerospike_scan_foreach(self->client->as, &err, scan_policy_p, &self->scan, each_result, &data);
 
 	PyEval_RestoreThread(_save);
 
-	if ( err.code != AEROSPIKE_OK ) {
+
+CLEANUP:
+
+	if (err.code != AEROSPIKE_OK) {
 		PyObject * py_err = NULL;
 		error_to_pyobject(&err, &py_err);
-		PyErr_SetObject(PyExc_Exception, py_err);
+		PyObject *exception_type = raise_exception(&err);
+		PyErr_SetObject(exception_type, py_err);
+		Py_DECREF(py_err);
 		return NULL;
 	}
 
